/***************************************************************************
 *   Copyright (C) 2004 by Danny Kukawka                                   *
 *                         <dkukawka@suse.de>, <danny.kukawka@web.de>      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of version 2 of the GNU General Public License     *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#ifndef AUTOSUSPEND_H
#define AUTOSUSPEND_H

// from project
#include "inactivity.h"

/*! 
*  \file 	autosuspend.h
*  \brief 	Headerfile for autosuspend.cpp and the class \ref autosuspend.
*/
/*! 
*  \class 	autosuspend
*  \brief 	class for autosuspend related funtionality
*  \author 	Danny Kukawka, <dkukawka@suse.de>, <danny.kukawka@web.de>
*  \date    	2005
*/

class autosuspend : public inactivity
{
	TQ_OBJECT
  
	
public:
	//! default constructor
	autosuspend(screen *disp);
	//! default destructor
	~autosuspend();
};

#endif
